@@ -39,7 +39,6 @@
 #include <IOKit/storage/IOMedia.h>
 #include <IOKit/IOBSD.h>
 
-#include "_psutil_osx.h"
 #include "_psutil_common.h"
 #include "arch/osx/process_info.h"
 
@@ -366,9 +365,11 @@ psutil_proc_memory_maps(PyObject *self, PyObject *args) {
 
             if (strlen(buf) == 0) {
                 switch (info.share_mode) {
+// #ifdef SM_LARGE_PAGE
                     // case SM_LARGE_PAGE:
                         // Treat SM_LARGE_PAGE the same as SM_PRIVATE
                         // since they are not shareable and are wired.
+// #endif
                     case SM_COW:
                         strcpy(buf, "[cow]");
                         break;
@@ -581,11 +582,14 @@ psutil_proc_memory_uss(PyObject *self, PyObject *args) {
     mach_msg_type_number_t info_count = VM_REGION_TOP_INFO_COUNT;
     kern_return_t kr;
     vm_size_t page_size;
+    mach_vm_address_t addr = MACH_VM_MIN_ADDRESS;
+    mach_port_t task = MACH_PORT_NULL;
+    vm_region_top_info_data_t info;
+    mach_port_t object_name;
 
     if (! PyArg_ParseTuple(args, "l", &pid))
         return NULL;
 
-    mach_port_t task = MACH_PORT_NULL;
     err = task_for_pid(mach_task_self(), pid, &task);
     if (err != KERN_SUCCESS) {
         psutil_raise_ad_or_nsp(pid);
@@ -600,10 +604,7 @@ psutil_proc_memory_uss(PyObject *self, PyObject *args) {
 
     // Roughly based on libtop_update_vm_regions in
     // http://www.opensource.apple.com/source/top/top-100.1.2/libtop.c
-    for (mach_vm_address_t addr = MACH_VM_MIN_ADDRESS; ; addr += size) {
-        vm_region_top_info_data_t info;
-        mach_port_t object_name;
-
+    for (addr = 0; ; addr += size) {
         kr = mach_vm_region(
             task, &addr, &size, VM_REGION_TOP_INFO, (vm_region_info_t)&info,
             &info_count, &object_name);
@@ -612,7 +613,8 @@ psutil_proc_memory_uss(PyObject *self, PyObject *args) {
             break;
         }
         else if (kr != KERN_SUCCESS) {
-            return false;
+            PyErr_Format(PyExc_RuntimeError, "mach_vm_region() failed");
+            return NULL;
         }
 
         if (psutil_in_shared_region(addr, cpu_type) &&
@@ -621,8 +623,10 @@ psutil_proc_memory_uss(PyObject *self, PyObject *args) {
         }
 
         switch (info.share_mode) {
+#ifdef SM_LARGE_PAGE
             case SM_LARGE_PAGE:
                 // NB: Large pages are not shareable and always resident.
+#endif
             case SM_PRIVATE:
                 private_pages += info.private_pages_resident;
                 private_pages += info.shared_pages_resident;
@@ -1783,6 +1787,36 @@ error:
 }
 
 
+/*
+ * Return CPU statistics.
+ */
+static PyObject *
+psutil_cpu_stats(PyObject *self, PyObject *args) {
+    struct vmmeter vmstat;
+    kern_return_t ret;
+    mach_msg_type_number_t count = sizeof(vmstat) / sizeof(integer_t);
+    mach_port_t mport = mach_host_self();
+
+    ret = host_statistics(mport, HOST_VM_INFO, (host_info_t)&vmstat, &count);
+    if (ret != KERN_SUCCESS) {
+        PyErr_Format(PyExc_RuntimeError,
+                     "host_statistics() failed: %s", mach_error_string(ret));
+        return NULL;
+    }
+    mach_port_deallocate(mach_task_self(), mport);
+
+    return Py_BuildValue(
+        "IIIII",
+        vmstat.v_swtch,  // ctx switches
+        vmstat.v_intr,  // interrupts
+        vmstat.v_soft,  // software interrupts
+        vmstat.v_syscall,  // syscalls
+        vmstat.v_trap  // traps
+    );
+}
+
+
+
 /*
  * define the psutil C module methods and initialize the module.
  */
@@ -1862,6 +1896,8 @@ PsutilMethods[] = {
      "Return dict of tuples of disks I/O information."},
     {"users", psutil_users, METH_VARARGS,
      "Return currently connected users as a list of tuples"},
+    {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
+     "Return CPU statistics"},
 
     {NULL, NULL, 0, NULL}
 };
